function resirf=msvar_irf(res,hor,mres)

// PURPOSE: provide the impulse response functions (irf) from a 
// Markov-switching VAR
// ------------------------------------------------------------
// INPUT:
// * res = a tlist result from a ms var estimation
// * hor = a scalar, the horizon of the irf (note that the
//   shocks are supposed to happen at time 0; therefore (hor+1)
//   impulses are calculated)
// * mres = optional argument that can be:
//   - x = chol1 (cholesky decomposition)
//   - x = chol2 (triangular factorisation)
//   - x = original (original residuals)
//     (default = chol1)
// ------------------------------------------------------------
// OUTPUT:
// resirf = a result tlist with:
// - resirf('meth') = 'msvar irf'
// - resirf('msvar res') = the input ms var results tlist
// - resirf('# irf') = the horizon of the irf calculations
// - resirf('nb states') = the # of states 
// - resirf('nb irf') = the # of different AR parts
//   * 1 if they do not switch
//   * the # states if they switcch
// - resirf('sigma switch') = the # of switching wariances
//   * 1 if they do not switch
//   * the # states if they switch
// 1) if both are equal to 1 then
// - resirf('irf') = the only set of impulse repsonse function,
//    a (nvar.hor x nvar) matrix:
//   [IRF(t=0)
//       .
//       .
//       .
//   IRF(t=hor)]
//   , with nvar= # of endogenous varaibles in the VAR
// 
// 2) if resirf('nb states') == 1 and resirf('sigma switch') ~= 1 
//   then:
// - resirf('irf state # 1') until resirf('irf state # ''nstates''')= 
//   the set of impulse repsonse function, each a (nvar.hor x nvar)
//   matrix:
//   [IRF(t=0)
//       .
//       .
//       .
//   IRF(t=hor)]
//   conditional of the state of the shock
// 
// 3) if resirf('nb states') == 1 and resirf('sigma switch') ~= 1 
//   then:
// - resirf('irf part state # 1') until 
//   resirf('irf part state # ''nstates''')= 
//   the set of impulse response function, each a (nvar.hor x nvar)
//   matrix:
//   [IRF(t=0)
//       .
//       .
//       .
//   IRF(t=hor)]
//   conditional of the state of the shock and the fact that
//   the state does not switch (not realistic, but useful to
//   the analysis of the VAR associated to each regime)
// - resirf('irf full state # 1') until 
//   resirf('irf fulle state # ''nstates''')= 
//   the set of full impulse response function, each a 
//   (nvar.hor x nvar) matrix:
//   [IRF(t=0)
//       .
//       .
//       .
//   IRF(t=hor)]
// ------------------------------------------------------------
// REFERENCE:
// Krolzig H-M., 2006, "Impulse-Response Analysis in Markov 
// Switching Vector Autoregressive Models," Economics 
// Department. University of Kent. Keynes College,
// http://f.karame.free.fr/IUP_IES/Var_Lat/Projets/Krolzig.pdf
// ------------------------------------------------------------
// Copyright: Eric Dubois 2011
// http://dubois.ensae.net/grocer.html

mres='chol2'
meth='asym'
niter=1000
siz=0.05

if res('meth') ~= 'ms var' then
   error('first arg should be a ms var results tlist')
end

for i=1:length(varargin)
   ieq=strindex(varargin(i),'=')
   car1=part(varargin(i),1:ieq-1)
   l=length(varargin(i))
 
   if car1 == 'mres' then
      mres=part(varargin(i),ieq+1:l)
   end
end
 
if and(meth ~= ['asym' ; 'mc1']) then
   error(meth+' is not an available method to compute ...
         confidence bands in irf')
end

beta_id=res('beta_id')
beta_co=res('beta_co')
nlag=res('nlag')
nendo=res('nendo')
sigma_switch=res('switching V')
nobs=res('nobs')
nb_states=res('nb_states')
resirf=tlist(['results';'meth';'msvar res';'# irf';'nb states';'# AR';'sigma switch'],...
'msvar irf',res,hor,nb_states,1-(nb_states-1)*(res('typemod')-3),sigma_switch)

if isempty(beta_co) then
   xmat=res('xmat')
   x=xmat(1:nobs,1:nendo*nlag)
   [xpxi]=invxpx(x)

   // prepare the full IRF
   PHI_1=[]
   ptrans=res('ptrans')
   for i=1:nb_states
      beta_i=beta_id(:,i)
      beta_const=beta_i((nendo*nlag+1)*[1:nendo])
      // remove constants from the beta vector
      beta_i((nendo*nlag+1)*[1:nendo])=[]
      beta_i=matrix(beta_i,nendo*nlag,nendo)'
      PHI_1=[PHI_1 ; [(ptrans(i,:) .*.ones(nlag*nendo,nlag*nendo)) .* ...
           (ones(1,nb_states) .*. [beta_i ; zeros(nendo*(nlag-1),nendo*(nlag-1)) eye(nendo,nendo)] ) ] ...
           (ptrans(i,:) .*. [beta_const ; zeros(nendo*(nlag-1),1)])]
    end  
    PHI=[PHI_1 ; zeros(nb_states,nendo*nlag*nb_states) ptrans]
    IRF_PSI = [eye(nb_states*(nendo*nlag+1),nb_states*(nendo*nlag+1)) ; zeros(hor*nb_states*(nendo*nlag+1),nb_states*(nendo*nlag+1))]
    IRF_yt_part = zeros(hor*nendo,(nendo*nlag+1)*nb_states)
    for j=1:hor
       PSI_j=PHI*IRF_PSI((j-1)*nb_states*(nendo*nlag+1)+1:j*nb_states*(nendo*nlag+1),:)
       IRF_PSI(j*nb_states*(nendo*nlag+1)+1:(j+1)*nb_states*(nendo*nlag+1),:)=PSI_j
       IRF_yt_part((j-1)*nendo+1:j*nendo,:)=[(ones(1,nb_states) .*. ([1 zeros(1,nlag-1)] .*. eye(nendo,nendo))) ...
                                             zeros(nendo,nb_states)]*PSI_j
    end

   for i=1:nb_states
      if sigma_switch == 1 then
         sigma=res('sigma')
      else
         sigma=res('sigma')
         sigma=sigma(:,nendo*(i-1)+1:i*nendo)
      end

      beta_i=beta_id(:,i)
      // remove constants from the beta vector
      beta_i((nendo*nlag+1)*[1:nendo])=[]
      beta_i=matrix(beta_i,nendo*nlag,nendo)

      select mres
      case 'chol1' then
         // Cholesky of omega = P'*P
         msg = 'Orthog. IRF: 1 sigma changes';
         P = chol(sigma)';
      case 'chol2' then
         // triangular fact. omega = A*D*A'
         msg = 'Orthog. IRF: 1 unit changes';
         cholsigmap    = chol(sigma)'
         Dsr = diag(diag(cholsigmap));
         P = inv(Dsr)*cholsigmap;
      else
         msg = 'Unorthog. IRF: 1 unit changes';
         P = eye(N,N);
      end
         
      [IRF_part,PHI]=irf0(beta_i,hor,nendo,nlag,P)

      z0=zeros(nendo*nlag,nendo)
      zi=[P ; zeros(nendo*(nlag-1),nendo)]
      zn=zeros(nb_states,nendo)
      
      IRF_full=[P ; IRF_yt_part*[ ones(i-1,1) .*. z0 ; zi ; ones(nb_states-i,1) .*. z0 ; zn]]
      
      resirf(1)($+1)='irf part state # '+string(i)
      resirf($+1)=IRF_part
      resirf(1)($+1)='irf full state # '+string(i)
      resirf($+1)=IRF_full

   end

else
   // case when AR terms do not switch
   x=res('zmat')
   [xpxi]=invxpx(x)
   beta_i=matrix(beta_co,nendo*nlag,nendo)

   if sigma_switch == 1 then
   // variances do not switch: the impulse response functions are not regime dependent
      sigma=res('sigma')

      select mres
      case 'chol1' then
         // Cholesky of omega = P'*P
         msg = 'Orthog. IRF: 1 sigma changes';
         P = chol(sigma)';
      case 'chol2' then
         // triangular fact. omega = A*D*A'
         msg = 'Orthog. IRF: 1 unit changes';
         cholsigmap    = chol(sigma)'
         Dsr = diag(diag(cholsigmap));
         P = inv(Dsr)*cholsigmap;
      else
         msg = 'Unorthog. IRF: 1 unit changes';
         P = eye(N,N);
      end

      [IRF,PHI]=irf0(beta_i,hor,nendo,nlag,P)
      resirf(1)($+1)='irf'
      resirf($+1)=IRF

   else
      for i=1:nb_states
         sigma=res('sigma')
         sigma=sigma(:,nendo*(i-1)+1:i*nendo)

         select mres
         case 'chol1' then
            // Cholesky of omega = P'*P
            msg = 'Orthog. IRF: 1 sigma changes';
            P = chol(sigma)';
         case 'chol2' then
            // triangular fact. omega = A*D*A'
            msg = 'Orthog. IRF: 1 unit changes';
            cholsigmap    = chol(sigma)'
            Dsr = diag(diag(cholsigmap));
            P = inv(Dsr)*cholsigmap;
         else
            msg = 'Unorthog. IRF: 1 unit changes';
            P = eye(N,N);
         end

         [IRF,PHI]=irf0(beta_i,hor,nendo,nlag,P)
         resirf(1)($+1)='irf state #'+string(i)
         resirf($+1)=IRF
      end
   end   

end

endfunction